/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sample.tswebclient.bean;

import java.io.IOException;
import java.io.OutputStream;
import javax.inject.Inject;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sample.tswebclient.cassandra.Application;
import sample.tswebclient.cassandra.CassandraManagerEJB;
import sample.tswebclient.cassandra.Attributes;
import sample.tswebclient.cassandra.ChildrenReadOption;

/**
 *
 * This servlet is used to download the rdp file when an application is clicked.
 * JSF are not very good when a downloadable binary file is needed, so when the user
 * clicks over the application this servlet is invoked and the RDP file
 * is downloaded. The content type returned is "application/x-rdp" in order
 * the browser calls Terminal Services client application.
 *
 * @author ricky
 */
public class FileDownloadServlet extends HttpServlet {

    private static final Logger logger = LoggerFactory.getLogger(FileDownloadServlet.class);

    @Inject
    private CassandraManagerEJB manager;

    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * The key application path is sent over "key" parameter, the application is read
     * using Cassandra EJB manager and the RDP file is downloaded if exists.
     * 
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        logger.info("Enetring processRequest");
        OutputStream out = null;
        try {
            String key = request.getParameter("key");
            logger.info("Key received: " + key);
            Application app = manager.readApplication(
                    key, ChildrenReadOption.NONE,
                    Attributes.name, Attributes.type, Attributes.rdp);
            if (app != null && app.getRdp() != null && app.getRdp().length > 0) {
                byte[] binary_data = app.getRdp();
                String filename = app.getName() + ".rdp";
                response.setContentType("application/x-rdp");
                response.setHeader("Content-Disposition", "attachment; filename=\"" + filename + "\"");
                response.setHeader("Pragma", "private");
                response.setHeader("cache-control", "private, must-revalidate");
                out = response.getOutputStream();
                out.write(binary_data);
                out.flush();
                response.flushBuffer();
            } else {
                throw new ServletException("Invalid file!");
            }
        } catch (Exception e) {
            logger.error("Error returning file.", e);
            if (e instanceof ServletException) {
                throw (ServletException) e;
            } else if (e instanceof IOException) {
                throw (IOException) e;
            } else {
                throw new ServletException(e);
            }
        } finally {
            if (out != null) {
                out.close();
            }
        }
    }

    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Download file servlet";
    }
}
