/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sample.tswebclient.cassandra;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.commons.pool.PoolableObjectFactory;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;

/**
 *
 * Commons pool object factory for Cassandra drift connections. This class
 * is used to perform a pooled connection against cassandra.
 *
 * @author ricky
 * 
 */
public class CassandraClientObjectFactory implements PoolableObjectFactory {

    /**
     * Cassandra host.
     */
    private String host = null;

    /**
     * Cassandra port.
     */
    private int port = -1;

    /**
     * Create a new Cassandra object factory for a given host and port.
     * 
     * @param host The Cassandra host.
     * @param port The Cassandra port.
     */
    public CassandraClientObjectFactory(String host, int port) {
        this.host = host;
        this.port = port;
    }

    /**
     * Return the Cassandra host for the factory.
     *
     * @return The Cassandra hostname.
     */
    public String getHost() {
        return host;
    }

    /**
     * Return the Cassandra port of the factory.
     *
     * @return The cassandra port.
     */
    public int getPort() {
        return port;
    }

    /**
     * Method to create a new thrift connection against Cassandra. The host
     * and port are used to create the socket.
     *
     * @return A new Cassandra.Client object.
     * @throws Exception Some error.
     */
    @Override
    public Object makeObject() throws Exception {
        TTransport tr = new TSocket(host, port);
        TProtocol proto = new TBinaryProtocol(tr);
        Cassandra.Client client = new Cassandra.Client(proto);
        tr.open();
        return client;
    }

    /**
     * Method to destroy a previously created Cassandra.Client object. The
     * transport of the client is got and closed.
     *
     * @param o The Casandra.Client to close.
     * @throws Exception Some error.
     */
    @Override
    public void destroyObject(Object o) throws Exception {
        Cassandra.Client client = (Cassandra.Client) o;
        TTransport tr = client.getOutputProtocol().getTransport();
        tr.close();
    }

    /**
     * Method to validate if a previously created Cassandra.Client is ok. The
     * method calls the describe_cluster cassandra method.
     *
     * @param o The Cassandra.Client to test.
     * @return True if ok, false if not.
     */
    @Override
    public boolean validateObject(Object o) {
        Cassandra.Client client = (Cassandra.Client) o;
        boolean validation = true;
        try {
            String cluster = client.describe_cluster_name();
            validation = cluster != null;
        } catch (Exception e) {
            validation = false;
        }
        return validation;
    }

    /**
     * N/A
     *
     * @param o
     * @throws Exception
     */
    @Override
    public void activateObject(Object o) throws Exception {
        // not needed
    }

    /**
     * N/A
     * 
     * @param o
     * @throws Exception
     */
    @Override
    public void passivateObject(Object o) throws Exception {
        // not needed
    }

}
